# SPDX-FileCopyrightText: 2025 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

sbom_compliance_tool_version = '0.0.2'
description = 'Tool to assists complioance work with SBOMs'
module_name = 'sbom_compliance_tool'



#!/bin/env python3

# SPDX-FileCopyrightText: 2025 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import argparse
import logging
import sys

from sbom_compliance_tool.sbom import SBoMReaderFactory
from sbom_compliance_tool.format import SBoMReportFormatterFactory

from licomp.interface import UseCase
from licomp.interface import Provisioning
from licomp.interface import Modification


def main():

    args = get_args()

    if args.debug:
        logging.getLogger().setLevel(logging.DEBUG)
    logging.debug("SBoM Compliance Tool")

    reader = SBoMReaderFactory.reader()
    logging.debug(f'Reader: {reader}')

    report = reader.check_file('example-data/normalized-project.json',
                               UseCase.usecase_to_string(UseCase.LIBRARY),
                               Provisioning.provisioning_to_string(Provisioning.BIN_DIST),
                               Modification.modification_to_string(Modification.UNMODIFIED))
    logging.debug(f'Report: {report}')

    formatter = SBoMReportFormatterFactory.formatter(args.output_format)
    formatted_report = formatter.format(report)

    print(formatted_report)

def get_parser():
    parser = argparse.ArgumentParser(prog="sbom-....",
                                     description="",
                                     epilog="",
                                     formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument('-of', '--output-format',
                        type=str,
                        default='json')

    parser.add_argument('-v', '--verbose',
                        action='store_true',
                        default=False)

    parser.add_argument('-d', '--debug',
                        action='store_true',
                        default=False)

    return parser

def get_args():
    return get_parser().parse_args()


if __name__ == '__main__':
    sys.exit(main())



# SPDX-FileCopyrightText: 2025 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import json

class SBoMReportFormatterFactory():

    @staticmethod
    def formatter(fmt):
        if fmt.lower() == 'markdown':
            return SBoMReportFormatterMarkdown()
        else:
            return SBoMReportFormatterJson()


class SBoMReportFormatter():

    def format(self, report):
        return None


class SBoMReportFormatterJson(SBoMReportFormatter):

    def format(self, report):
        return json.dumps(report, indent=4)

class SBoMReportFormatterMarkdown(SBoMReportFormatter):

    def format(self, report):
        lines = []

        lines.append('# Compliance report')
        lines.append('')
        lines.append('## Summary')
        lines.append(f'* name: {report["name"]}')
        lines.append(f'* version: {report["version"]}')
        lines.append(f'* otbound license: {report["license"]}')
        lines.append(f'* compatibility: {report["compatibility"]}')
        lines.append('')
        lines.append('## Details')
        lines.append('')
        lines.append('### Dependencies ')
        for dep in report['dependencies']:
            lines.append('')
            lines.append(f'#### {dep["name"]}')
            lines.append('')
            lines.append(f'* version: {dep["version"]}')
            lines.append(f'* license: {dep["license"]}')
            lines.append(f'* compatibility: {dep["compatibility"]}')

        return "\n".join(lines)



# SPDX-FileCopyrightText: 2025 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import json

from licomp_toolkit.toolkit import ExpressionExpressionChecker

class SBoMReaderFactory():

    @staticmethod
    def reader():
        return SBoMReader()

class SBoMReader():

    def __init__(self):
        pass

    def __read(self, file_name):
        with open(file_name) as fp:
            return json.load(fp)

    def update_compat(self, current, new):
        _map = {
            None: 0,
            'yes': 1,
            'mixed': 2,
            'depends': 3,
            'unsupported': 4,
            'no': 5,
        }
        p_current = _map[current]
        p_new = _map[new]
        if p_new > p_current:
            return new
        return current

    def check_data(self, sbom_content, usecase, provisioning, modified):
        outbound = sbom_content["license"]
        report = {
            'name': sbom_content["name"],
            'version': sbom_content["version"],
            'license': outbound,
        }

        resources = ['licomp_reclicense', 'licomp_osadl', 'licomp_proprietary']
        compat_checker = ExpressionExpressionChecker()
        deps = []
        top_compat = None
        for dep in sbom_content["dependencies"]:
            inbound = dep["license"]
            dep_compat = compat_checker.check_compatibility(outbound,
                                                            inbound,
                                                            usecase,
                                                            provisioning,
                                                            resources)

            new_dep = dep.copy()
            compat = dep_compat['compatibility']
            new_dep['compatibility'] = compat
            new_dep['compatibility_details'] = dep_compat
            deps.append(new_dep)
            top_compat = self.update_compat(top_compat, compat)

        report['compatibility'] = top_compat
        report['dependencies'] = deps

        return report

    def check_file(self, file_name, usecase, provisioning, modified):
        with open(file_name) as fp:
            return self.check_data(json.load(fp), usecase, provisioning, modified)



# SPDX-FileCopyrightText: 2025 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import setuptools

from sbom_compliance_tool.config import sbom_compliance_tool_version
from sbom_compliance_tool.config import description
from sbom_compliance_tool.config import module_name

requirements = []
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

requirements_dev = []
with open('requirements-dev.txt') as f:
    requirements_dev = f.read().splitlines()

with open("README.md") as f:
    _long_description = f.read()

setuptools.setup(
    name='sbom_compliance_tool',
    version=sbom_compliance_tool_version,
    author="Henrik Sanklef",
    author_email="hesa@sandklef.com",
    description=description.replace('\n', ' '),
    long_description=_long_description,
    long_description_content_type="text/markdown",
    license_files=('LICENSES/GPL-3.0-or-later.txt',),
    url="https://github.com/hesa/licomp-toolkit",
    packages=['sbom_compliance_tool'],
    entry_points={
        "console_scripts": [
            "licomp-toolkit = sbom_compliance_tool.__main__:main",
        ],
    },
    package_data={
        f'{module_name}': ['data/*.json'],
    },
    install_requires=requirements,
    extras_require={
        'dev': requirements_dev,
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Legal Industry",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Quality Assurance",
    ],
    python_requires='>=3.6',
)
